#!/usr/bin/perl -wT
#
# autopsyfunc.pm
# Autopsy Forensics Browser
#
# Provides backend functionality to autopsy
#
# Brian Carrier [carrier@cerias.purdue.edu]
#
# This file requires The Coroners Toolkit (TCT)
#    www.fish.com/tct/
#
# and TCTUTILs
#	www.cerias.purdue.edu/homes/carrier/forensics
#
#  $Revision: 0.2 $
# 
# Copyright (c) 2001 by Brian Carrier.  All rights reserved
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# 3. The name of the author may not be used to endorse or promote
#    products derived from this software without specific prior written
#    permission.
#
#
# THIS SOFTWARE IS NOT AFFILIATED WITH PURDUE UNIVERSITY OR THE CENTER FOR
# EDUCATION IN INFORMATION ASSURANCE AND SECURITY (CERIAS) AND THEY BEAR
# NO RESPONSIBILITY FOR ITS USE OR MISUSE.
#
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR ANY PARTICULAR PURPOSE.
#
# IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, LOSS OF USE, DATA, OR PROFITS OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

# See HELP section for listing of arguments and required files
use lib './';
use integer;
use strict;
require 'conf.pl';

# Get rid of unsecure settings
$ENV{PATH} = "";
delete @ENV{ 'IFS', 'CDPATH', 'ENV', 'BASH_ENV'};

# import variables from conf.pl
use vars '$PROGNAME', '$TCTDIR', '$MORGDIR', '$TCTUTILDIR';
use vars '$NAME', '$VER', '$GREP_EXE', '$MD5_EXE', '$STRINGS_EXE';
use vars '$INVESTIG', '$HOME_PAGE';

# Formats for regular expressions
my $REG_DATE = '\d\d.\d\d.\d\d\d\d \d\d\:\d\d\:\d\d';

# constants for different functions
my $MENU=0;
my $HELP=1;
my $BLANK=2;

my $FIL_MAIN=3;
my $FIL_DIR=4;
my $FIL_LIST=5;
my $FIL_CONT=6;
my $FIL_REP=7;
my $FIL_CIDXF=8;
my $FIL_CIDX=9;
my $FIL_DEL=10;

my $BLK_ENT=11;
my $BLK_MAIN=12;
my $BLK_LIST=13;
my $BLK_CONT=14;
my $BLK_REP=15;
my $BLK_CIDXF=16;
my $BLK_CIDX=17;

my $SRCH_ENT=18;
my $SRCH_MAIN=19;
my $SRCH_LIST=20;

my $INO_ENT=21;
my $INO_MAIN=22;
my $INO_LIST=23;
my $INO_CONT=24;
my $INO_REP=25;


# Methods of sorting the file listings
my $SORT_TYPE=0;
my $SORT_NAME=1;
my $SORT_MOD=2;
my $SORT_ACC=3;
my $SORT_CRE=4;
my $SORT_SIZE=5;
my $SORT_GID=6;
my $SORT_UID=7;
my $SORT_INODE=8;
my $SORT_DEL=9;

# Display types that use the sort variable 
my $FIL_SORT_ASC=0;
my $FIL_SORT_STR=1;

my $BLK_SORT_ASC=0;
my $BLK_SORT_HEX=1;
my $BLK_SORT_STR=2;


my @funcs;		# Array for function references
my %args;		# Arguments that are decoded
my %enc_args;	# Arguments that are encoded
my $baseargs;	# Arguments that never change such as rdir and img


# Main Menu
$funcs[$MENU] = sub {
	if (check_fsmorgue()) {
		print_config();
		return 1;
	}

	print "<HEAD><TITLE>AUTOPSY: Main Menu</TITLE></HEAD>\n".
		"<BODY><CENTER><H1>Autopsy Forensics Browser </H1>".
		"<H2>ver $VER</H2>\n";

	# Browsing Type
	print "<FORM ACTION=\"$PROGNAME\" METHOD=\"GET\">\n".
		"<HR><BR>Browsing Type:<BR><BR>\n".
		"Files <INPUT TYPE=\"radio\" NAME=\"func\" VALUE=$FIL_MAIN ".
		"CHECKED><BR>\n".
		"Blocks <INPUT TYPE=\"radio\" NAME=\"func\" VALUE=$BLK_MAIN><BR>\n".
		"Inode <INPUT TYPE=\"radio\" NAME=\"func\" VALUE=$INO_MAIN><BR>\n".
		"Search <INPUT TYPE=\"radio\" NAME=\"func\" VALUE=$SRCH_ENT><BR>\n".
		"<INPUT TYPE=\"hidden\" NAME=\"dir\" VALUE=\"\">\n".
		"<INPUT TYPE=\"hidden\" NAME=\"inode\" VALUE=\"2\">\n<BR><HR>\n";

	# Mounting Point and image
	#
	# fsmorgue file should have listings such as
	# my_dded_file1	/usr
	#

	print "<BR>image<BR>(select both boxes)<BR><BR>\n";
	unless (open (FILE, $MORGDIR.'fsmorgue')) { 
		print "Error opening $MORGDIR.fsmorgue<BR>\n";
		return 1;
	}
	while (<FILE>) {
		if ((/^\#/) || (/^\s+$/)) {
			next;
		}
		elsif (/^([\w\-\_\.]+)\s+([\w\-\_\.\/]+)$/) {
			my $a = $1;
			unless (-e $MORGDIR.$a) {
				print "Error: file not found: ".$MORGDIR.$a."<BR>\n";
				next;
			}

			# Add trailing / if needed
			my $b = $2;
			if ($b !~ /.*?\/$/) {
				$b .= '/';
			}

			print "$a on $b<INPUT TYPE=\"radio\" NAME=\"img\" VALUE=$a>".
			  "<INPUT TYPE=\"radio\" NAME=\"rdir\" VALUE=$b><BR>\n";
		}
		else {
			print "Error: invalid entry in fsmorgue on line $.<BR>\n";
		}
	}
	close (FILE);

	print "<HR>\n";
	# Get time zone difference 
	my $found = 0;
	if (-e $MORGDIR.'zoneinfo') {
		unless (open (FILE, $MORGDIR.'zoneinfo')) { 
			print "Error opening $MORGDIR/zoneinfo<BR>\n";
			return 1;
		}
		while (<FILE>) {
			if (/^\#/) {
				next;
			}
			elsif (/^([-\+]?\d+)$/) {
				print "<INPUT TYPE=\"hidden\" NAME=\"hour\" VALUE=\"$1\">\n";
				print "Time Zone difference: $1 hours<BR>";
				$found = 1;
				last;
			}
		}
		close(FILE);
	}

	if ($found == 0) {
		my $hr = 0;
		$hr = $args{'hour'} if (exists $args{'hour'});

		print "Time Zone difference: <INPUT TYPE=\"TEXT\" VALUE=\"$hr\" ".
		  "NAME=\"hour\" SIZE=\"5\"><BR>\n";
	}

	print "<INPUT TYPE=\"SUBMIT\" VALUE=\"Enter the Lab\">\n</FORM>\n";

	print "<HR><A HREF=\"$PROGNAME?func=$HELP\">".
		"<FONT SIZE=3>Help</A><BR></FONT>\n";

	return 0;
};

# Blank Page
$funcs[$BLANK] = sub { 
	print "<!-- This Page Intentionally Left Blank -->\n";
	return 0;
};


# print the current configuration
# used when displaying errors about bad arguments and for help
#
sub print_config {
	print "<HR><CENTER><h2>Current Configuration</H2></CENTER>".
		"<B>Version</B>: $VER<BR>".
		"<B>Morgue Directory</B>: $MORGDIR<BR>".
		"<B>TCT exec directory</B>: $TCTDIR<BR>".
		"<B>TCTUTILs exec directory</B>: $TCTUTILDIR<BR>".
		"<B>MD5 exec</B>: $MD5_EXE<BR>".
		"<B>strings exec</B>: $STRINGS_EXE<BR>".
		"<B>grep exec</B>: $GREP_EXE<BR>".
		"<B>Investigator</B>: $INVESTIG<BR>";
}

#
# HELP File
#
$funcs[$HELP] = sub {

print "<HR><CENTER><H1><A HREF=$HOME_PAGE>$NAME</A> Help Page</H1></CENTER>\n";

print_config();

print <<EOF;
<HR>
<CENTER><h2>General</H2></CENTER>
<P>

The $NAME Forensics Browser can be used to browse disk images
generated by using 'dd(1)'.  The browser requires the following
programs:

<UL>
<LI>The Coroners Toolkit (TCT) 
  [<A HREF=http://www.fish.com/tct>www.fish.com/tct</A>]
<LI> TCTUTILs [<A HREF=$HOME_PAGE>$HOME_PAGE</A>]
</UL>
</P>

<P> 
The images to analyze must be placed in the Morgue directory, which
can be changed by editing the \$MORGDIR variable in conf.pl or on
the command line of autopsy (currently set to: $MORGDIR).  The
morgue directory must contain a file called <i>fsmorgue</i> that
contains a list of images in the directory.  The format is the image
name, a tab, and where the image was mounted on the compromised
system (ie /usr/).  The image names can only have letters, numbers,
'-', '_', and '.' in them. An optional file, <i>zoneinfo</i>, 
can contain a single integer that specifies the number of hours
difference between the current site and the compromise site (to
adjust the time printing).  For example, when analyzing a system
in EST (GMT-5) that was from CST (GMT-6), <I>zoneinfo</I> file would 
contain -1.
</P>

<P>
The browser has the following main functions:
<UL>

<LI><B>File Browsing</B>:  Allows browsing the image as a file
system.  This gives a list of directories on the left, and files
and file content on the right hand side.  The output of each file
can be seen as ASCII or can be run through the <i>strings(1)</i>
utility if it exists.  Since this analyzes directory inode entries,
deleted file names can still be seen and depending on the OS, the
deleted file contents can also be easily recovered.  If a file name
has a '*' before it, it has been deleted.  The directory contents
frame can be resorted based on name, time, owner etc. by selecting
the appropriate column header.

<LI><B>Inode Browsing</B>: Allows browsing by inode number.  Enter
an inode number and see the details of the entry.  The file(s) that
are using the file will also be displayed (even if they have been
deleted for some OSes).  Inode browsing can also be accessed when
File browsing, by selecting the inode number for a file.  The blocks
that the inode has allocated can be selected when inode browsing
and will transfer you to block browsing.

<LI><B>Block Browsing</B>: Allows browsing by block number.  This
is most useful when used with searching or inode browsing.  The
contents of the block can be displayed in ASCII, hex dump, or through
<I>strings(1)</I>.  The inode that has allocated the block will be
displayed (if any) along with the file name (if any).

<LI><B>Block Searching</B>:  Search an image using <I>grep(1)</I>
for a given string.  The result will be a list of blocks that have
this string.  Selecting each block brings the user into Block
browsing mode to view the contents in a variety of formats.

<LI><B>Report Generation</B>:  Each of the above browsing techniques
allows a report to be generated.  This report lists the date, md5
value, investigator, and other context information in a text format.
This can be used for record keeping when deleted blocks of data
have been found.

</UL>
<HR>
<CENTER><h2>Arguments</H2></CENTER>
<H3>Required</H3>
<UL>

<LI>func: integer value that decides what function do to.  This
value is used as an index into the dirs array to call the correct
function.  These are defined as constants in this script, such
as \$FIL_MAIN.

<LI>img: name of the dd(1) image that resides in $MORGDIR. 

<LI>rdir: The mounting point of the image, such as /usr/ (or
%2Fusr%2f).  This must end in a '/'.

<LI>hour: an integer specifying how many hours difference the local
time is compared to the compromised systems time zone.  This will
make the time outputs the same as would be seen on the actual
machine.  This can be set with the zoneinfo zoneinfo file in $MORGDIR

</UL>

<H3>Optional</H3>
<UL>
<LI>inode: the inode to process for FIL_* and INO_* functions

<LI>dir: the directory being looked at for FIL_LIST or the file
being looked at for other FIL_* functions.  This must not begin
with a '/'.  It will be appended to the rdir argument

<LI>block: the block to process for FIL_* functions

<LI>str: the string to search for in SRCH_* functions

</UL>
EOF

}; #end of HELP


#
###################### FILE ###################
#

#
# Make the three frames and fill them in
#
$funcs[$FIL_MAIN] = sub {
	if (check_inode() || check_img() || check_dir() || check_rdir()) {
		print_config();
		return 1;
	}

	my $url;
	my $sort;
	if (exists $args{'sort'}) {
		$sort = $args{'sort'};
	}
	else {
		$sort = $SORT_TYPE;
	}

	print "<HEAD><TITLE>$args{'rdir'}$args{'dir'} on $args{'img'}".
		"</TITLE></HEAD><FRAMESET COLS=\"20%,80%\">\n";

	# Directory Listings
	$url = "$PROGNAME?func=$FIL_DIR&$baseargs&sort=$sort".
	  "&dir=$enc_args{'dir'}";
	print "<FRAME SRC=\"$url\">\n";

	# File frameset
	print "<FRAMESET ROWS=\"50%,50%\">\n";

	# File Listings
	$url = "$PROGNAME?func=$FIL_LIST&$baseargs&".
		"dir=$enc_args{'dir'}&sort=$sort&inode=$enc_args{'inode'}";
	
	print "<FRAME SRC=\"$url\" NAME=\"list\">\n";

	# File Contents
	print "<FRAME SRC=\"$PROGNAME?func=$BLANK&$baseargs\" NAME=\"content\">\n".
		"</FRAMESET></FRAMESET>\n";

	return 0;
};

#
# Print the directory names for the lhs frame
#
$funcs[$FIL_DIR] = sub {
	if (check_img() || check_dir() || check_fls() || check_rdir() || 
	  check_sort()) {
		print_config();
		return 1;
	}

	print_index();

	my $url = "$PROGNAME?func=$FIL_DEL&$baseargs&inode=2&dir=".
		"&sort=$enc_args{'sort'}";

	print "<A HREF=\"$url\" TARGET=\"list\">All Deleted Files</A><BR>\n";

	# Print root 
	$url = "$PROGNAME?func=$FIL_LIST&$baseargs&inode=2&dir=".
		"&sort=$enc_args{'sort'}";

	print "<A HREF=\"$url\" TARGET=\"list\">$args{'rdir'}</A><BR>\n";

	my $lcldir = "";
	my $prev = ""; 			# previous number of '+' directory spacers
	my $preventry = "";
	my $img = get_img();
	my $hr = get_hour();

	foreach $_ (`${TCTUTILDIR}fls -urD -z $hr $img 2`) {
		if (/^\*?(\+*)\s*d\s*(\d+):\t(.+)$/) {
	
			# Adjust the dir value using the '++' values to determine
			# how "deep" we are
			unless ($prev eq $1) {
				if ($1 eq $prev.'+') {
					$lcldir .= ($preventry."/");
				}
				elsif (defined $1) {
					my @dirs = split ('/', $lcldir); 
					my $idx = -1;
					$lcldir = "";

					while (($idx = index ($1, '+', $idx+1)) != -1) {
						$lcldir .= ($dirs[$idx]."/");
					}
				}
			}
			$prev = $1;
			$preventry = $3;

			$url = "$PROGNAME?func=$FIL_LIST&$baseargs&inode=$2&".
				"&dir=".url_encode($lcldir.$3."/").
				"&sort=$enc_args{'sort'}";

			print "+$1<A HREF=\"$url\" TARGET=\"list\">/$3</A><BR>\n";
		}
	}
	return 0;

};	# end of FIL_DIR


# Print the files and directories for the upper rhs frame
# These can be sorted in any format
$funcs[$FIL_LIST] = sub {
	if (check_img() || check_inode() || check_dir() || check_fls() ||
	  check_sort() || check_rdir()) {
		print_config();
		return 1;
	}

	my (@type, @name, @mod, @acc, @cre, @size, @gid, @uid, @inode, @dir);
	my (@entry, @del);

	my $sort = $args{'sort'};

	print "<HEAD><TITLE>Entries in $args{'rdir'}$args{'dir'}</TITLE></HEAD>\n";
	print "<H3>Entries in $args{'rdir'}$args{'dir'} </H3>\n";
	
	# Make the Table and Headers
	my $url="$PROGNAME?func=$FIL_MAIN&$baseargs&inode=$enc_args{'inode'}".
		"&dir=$enc_args{'dir'}";

	my $iurl;
	print "<TABLE>\n<TR>\n";

	if ($sort == $SORT_DEL) {
		print "<TH>d</TH>\n";
	} else {
		$iurl = $url."&sort=$SORT_DEL";
		print "<TH><A HREF=\"$iurl\" TARGET=\"_parent\">d</A></TH>\n";
	}

	if ($sort == $SORT_TYPE) {
		print "<TH>tp</TH>\n";
	} else {
		$iurl = $url."&sort=$SORT_TYPE";
		print "<TH><A HREF=\"$iurl\" TARGET=\"_parent\">tp</A></TH>\n";
	}

	if ($sort == $SORT_NAME) {
		print "<TH>Name</TH>\n";
	} else {
		$iurl = $url."&sort=$SORT_NAME";
		print "<TH><A HREF=\"$iurl\" TARGET=\"_parent\">Name</A></TH>\n";
	}

	if ($sort == $SORT_MOD) {
		print "<TH>Modified</TH>\n";
	} else {
		$iurl = $url."&sort=$SORT_MOD";
		print "<TH><A HREF=\"$iurl\" TARGET=\"_parent\">Modified</A></TH>\n";
	}

	if ($sort == $SORT_ACC) {
		print "<TH>Accessed</TH>\n";
	} else {
		$iurl = $url."&sort=$SORT_ACC";
		print "<TH><A HREF=\"$iurl\" TARGET=\"_parent\">Accessed</A></TH>\n";
	}

	if ($sort == $SORT_CRE) {
		print "<TH>Created</TH>\n";
	} else {
		$iurl = $url."&sort=$SORT_CRE";
		print "<TH><A HREF=\"$iurl\" TARGET=\"_parent\">Created</A></TH>\n";
	}

	if ($sort == $SORT_SIZE) {
		print "<TH>Size</TH>\n";
	} else {
		$iurl = $url."&sort=$SORT_SIZE";
		print "<TH><A HREF=\"$iurl\" TARGET=\"_parent\">Size</A></TH>\n";
	}
	
	if ($sort == $SORT_UID) {
		print "<TH>UID</TH>\n";
	} else {
		$iurl = $url."&sort=$SORT_UID";
		print "<TH><A HREF=\"$iurl\" TARGET=\"_parent\">UID</A></TH>\n";
	}

	if ($sort == $SORT_GID) {
		print "<TH>GID</TH>\n";
	} else {
		$iurl = $url."&sort=$SORT_GID";
		print "<TH><A HREF=\"$iurl\" TARGET=\"_parent\">GID</A></TH>\n";
	}

	if ($sort == $SORT_INODE) {
		print "<TH>Inode</TH>\n";
	} else {
		$iurl = $url."&sort=$SORT_INODE";
		print "<TH><A HREF=\"$iurl\" TARGET=\"_parent\">Inode</A></TH></TR>\n";
	}

	my $cnt = 0;
	my $inode = get_inode();
	my $img = get_img();
	my $hr = get_hour();

	# Read each entry into arrays so we can sort them after
	#

	foreach $_ (`${TCTUTILDIR}fls -la -z $hr $img $inode`) {
		if (/^([\?dfcbrlsw])\s*(\*?)\s*(\d+):\t(.+?)\t($REG_DATE)\t($REG_DATE)\t($REG_DATE)\t(\d+)\t(\d+)\t(\d+)$/) {

			my $lcldir = $args{'dir'};
			$type[$cnt] = $1;
			$del[$cnt] = $2;
			$inode[$cnt] = $3;
			$name[$cnt] = $4;
			$mod[$cnt] = $5;
			$acc[$cnt] = $6;
			$cre[$cnt] = $7;
			$size[$cnt] = $8;
			$gid[$cnt] = $9;
			$uid[$cnt] = $10;

			# We must adjust the dir for directories
			if ($1 eq 'd') {
				if ($4 eq '..') {
					my @dirs = split ('/', $lcldir);
					my $i;
					$lcldir = "";
					for ($i = 0; $i < $#dirs; $i++) {
						$lcldir .= ($dirs[$i].'/');
					}
				}
				elsif ($4 ne '.') {
					$lcldir .= ($4.'/');
				}
				$name[$cnt] .= '/';
			}
			else {
				$lcldir .= $4;
			}

			$dir[$cnt] = url_encode($lcldir);
			$entry[$cnt] = $cnt;
			$cnt++;

		} else {
			print "<TR><TD COLSPAN=10>FILE $_</TD></TR>\n";
		}
	}

	my @sorted;
	if ($sort == $SORT_TYPE) {
		@sorted = sort {$type[$a] cmp $type[$b]} @entry;
	} elsif ($sort == $SORT_NAME) {
		@sorted = sort {$name[$a] cmp $name[$b]} @entry;
	} elsif ($sort == $SORT_MOD) {
		@sorted = sort {$mod[$a] cmp $mod[$b]} @entry;
	} elsif ($sort == $SORT_ACC) {
		@sorted = sort {$acc[$a] cmp $acc[$b]} @entry;
	} elsif ($sort == $SORT_CRE) {
		@sorted = sort {$cre[$a] cmp $cre[$b]} @entry;
	} elsif ($sort == $SORT_SIZE) {
		@sorted = sort {$size[$a] <=> $size[$b]} @entry;
	} elsif ($sort == $SORT_UID) {
		@sorted = sort {$uid[$a] <=> $uid[$b]} @entry;
	} elsif ($sort == $SORT_GID) {
		@sorted = sort {$gid[$a] <=> $gid[$b]} @entry;
	} elsif ($sort == $SORT_INODE) {
		@sorted = sort {$inode[$a] <=> $inode[$b]} @entry;
	} elsif ($sort == $SORT_DEL) {
		@sorted = reverse sort {$del[$a] cmp $del[$b]} @entry;
	}

	foreach my $i (@sorted) {
		my $url;
		my $target;
		my $func;


		if ($type[$i] eq 'd') {
			$target = "list";
			$url="$PROGNAME?func=$FIL_LIST&$baseargs&inode=$inode[$i]".
				"&sort=$enc_args{'sort'}&dir=$dir[$i]";
		}
		else {
			$target = "content";
			$url="$PROGNAME?func=$FIL_CIDXF&$baseargs&inode=$inode[$i]".
				"&sort=$enc_args{'sort'}&dir=$dir[$i]";
		}

		my $iurl="$PROGNAME?func=$INO_CONT&$baseargs&inode=$inode[$i]";

		print "<TR><TD>$del[$i]</TD><TD>$type[$i]</TD>";
		if ($inode[$i] > 1) {
			print "<TD><A HREF=\"$url\" TARGET=\"$target\">$name[$i]</A></TD>";
		} else {
			print "<TD>$name[$i]</TD>";
		}

		print "<TD>$mod[$i]</TD><TD>$acc[$i]</TD><TD>$cre[$i]</TD>".
			"<TD>$size[$i]</TD><TD>$gid[$i]</TD><TD>$uid[$i]</TD>";
		if ($inode[$i] > 1) {
			print "<TD><A HREF=\"$iurl\" TARGET=\"content\">$inode[$i]</A></TD>";
		} else {
			print "<TD>$inode[$i]</TD>";
		}
		print "</TR>\n";

	}

	print "</TABLE>\n";
	return 0;

}; #end of FIL_LIST


# Content Frame
# This creates two frames for the lower rhs frame
#
$funcs[$FIL_CIDXF] = sub {
	if (check_img() || check_inode() || check_dir() ||
	  check_rdir()) {
		print_config();
		return 1;
	}

	print "<FRAMESET ROWS=\"10%,90%\">\n";

	print "<FRAME SRC=\"$PROGNAME?func=$FIL_CIDX&$baseargs".
	  "&dir=$enc_args{'dir'}&inode=$enc_args{'inode'}".
	  "&sort=$FIL_SORT_ASC\">\n";

	print "<FRAME SRC=\"$PROGNAME?func=$FIL_CONT&$baseargs".
	  "&dir=$enc_args{'dir'}&inode=$enc_args{'inode'}".
	  "&sort=$FIL_SORT_ASC\" NAME=\"cont2\">\n</FRAMESET>";

	return 0;
};


# This is the index for the lower rhs frame
# Choose the content display type here
$funcs[$FIL_CIDX] = sub {
	if (check_img() || check_inode() || check_dir() ||
	  check_rdir() || check_sort()) {
		print_config();
		return 1;
	}

	print "<CENTER>$args{'rdir'}$args{'dir'} : ";
	my $url = "&$baseargs&dir=$enc_args{'dir'}&inode=$enc_args{'inode'}";

	print "<A HREF=\"$PROGNAME?func=$FIL_CONT&sort=$FIL_SORT_ASC$url\"".
 	  "TARGET=\"cont2\"> ascii</A> ".
	  "<A HREF=\"$PROGNAME?func=$FIL_REP&sort=$FIL_SORT_ASC$url\"".
	  "TARGET=\"_blank\">".
 	  " (report)</A> - ".
	  "<A HREF=\"$PROGNAME?func=$FIL_CONT&sort=$FIL_SORT_STR$url\"".
 	  "TARGET=\"cont2\"> strings</A> ".
	  "<A HREF=\"$PROGNAME?func=$FIL_REP&sort=$FIL_SORT_STR$url\"".
	  "TARGET=\"_blank\">".
 	  " (report)</A></CENTER>";

	return 0;
};

# display deleted files only
$funcs[$FIL_DEL] = sub {

	if (check_img() || check_inode() || check_dir() || check_sort() ||
	  check_rdir() || check_fls() ) {
		print_config();
		return 1;
	}

	my $inode = get_inode();
	my $img = get_img();
	my $hr = get_hour();

	print "<HEAD><TITLE>Deleted files on $args{'img'} $args{'rdir'}</TITLE>".
	  "</HEAD>\n<TABLE>\n";

	print "<TR><TH>tp</TH><TH>Name</TH><TH>Modified</TH>".
	  "<TH>Accessed</TH><TH>Created</TH><TH>Size</TH><TH>UID</TH><TH>GID</TH>".
	  "<TH>Inode</TH></TR>\n";

	foreach $_ (`${TCTUTILDIR}fls -ldr -z $hr $img $inode`) {

		if (/^([\?dfcbrlsw])\s*(\*?)\s*(\d+):\t(.+?)\t($REG_DATE)\t($REG_DATE)\t($REG_DATE)\t(\d+)\t(\d+)\t(\d+)$/) {

			# We have to remove the / from the begining of the file name so
			# save all values so they aren't lost
			my $t = $1;
			my $d = $2;
			my $i = $3;
			my $n = $4;
			my $m = $5;
			my $a = $6;
			my $c = $7;
			my $s = $8;
			my $g = $9; 
			my $u = $10;

			if ($n =~ /^\/(.*)/) {
				$n = $1;
			}
			my $iurl="$PROGNAME?func=$INO_CONT&$baseargs&inode=$i".
				"&sort=$enc_args{'sort'}&dir=$n";

			print "<TR><TD>$t</TD>";
			if ($t eq 'd') {
				my $url="$PROGNAME?func=$FIL_LIST&$baseargs&inode=$i".
					"&sort=$enc_args{'sort'}&dir=$n";

				if ($i > 1) {
					print "<TD><A HREF=\"$url\" target=\"_self\">".
					  "$args{'rdir'}$n</A></TD>";
				} else {
					print "<TD>$args{'rdir'}$n</TD>";
				}
				
			}
			else {
				my $url="$PROGNAME?func=$FIL_CIDXF&$baseargs&inode=$i".
					"&sort=$enc_args{'sort'}&dir=$n";
				
				if ($i > 1) {
					print "<TD><A HREF=\"$url\" target=\"content\">".
					  "$args{'rdir'}$n</A></TD>";
				} else {
					print "<TD>$args{'rdir'}$n</TD>";
				}
			}

			print "<TD>$m</TD><TD>$a</TD><TD>$c</TD><TD>$s</TD><TD>$g</TD>".
			  "<TD>$u</TD>";
			if ($i > 1) {
				print "<TD><A HREF=\"$iurl\" target=\"content\">$i</A></TD></TR>\n";
			} else {
				print "<TD>$i</TD>\n";
			}
		}
		else {
			print "MISS: $_\n<BR>";
		}
	}
	print "</TABLE>\n";
};


#
# Display the actual content here
#
# NOTE: This has a media type of raw text
#
$funcs[$FIL_CONT] = sub {
	if (check_img() || check_inode() || check_dir() || check_sort() ||
	  check_rdir()) {
		return 1;
	}
	elsif ((($args{'sort'} == $FIL_SORT_ASC) && (check_icat())) ||
	  (($args{'sort'} == $FIL_SORT_STR) && (check_strings()))) {
		return 1;
	}

	my $inode = get_inode();
	my $img = get_img();

	if ($args{'sort'} == $FIL_SORT_ASC) {
		print "Contents Of File: $args{'rdir'}$args{'dir'}\n\n\n\n";
		print_output (`${TCTDIR}icat -H $img $inode`);
	} elsif ($args{'sort'} == $FIL_SORT_STR) {
		print "String Contents Of File: $args{'rdir'}$args{'dir'}\n\n\n\n";
		print_output (`${TCTDIR}icat -H $img $inode | $STRINGS_EXE`);
	}

	return 0;
};


# Display a report for a file
# This is intended to have its own window
#
$funcs[$FIL_REP] = sub {
	if (check_img() || check_inode() || check_dir() || check_rdir() ||
	  check_istat() || check_sort() ) {
		return 1;
	} 
	elsif ((($args{'sort'} == $FIL_SORT_ASC) && (check_icat())) ||
	  (($args{'sort'} == $FIL_SORT_STR) && (check_strings()))) {
		return 1;
	}
	my $sort = $args{'sort'};
	my $img = get_img();
	my $inode = get_inode();
	my $hr = get_hour();
	my $type;

	if ($sort == $FIL_SORT_ASC) {
		$type = "ascii";
	}elsif ($sort == $FIL_SORT_STR) {
		$type = "string";
	} else {
		print "invalid sort value";
		print_config();
		return 1;
	}

	# NOTE: There is a space in the beginning of the separator lines in
	# order to make clear@stamper.itconsult.co.uk time stamping happy
	# I think it confuses the lines that begin at the lhs with PGP 
	# headers and will remove the second line.
	#
	print "           $NAME $type Report (ver $VER)\n\n".
		  " -----------------------------------------------------\n".
		"File: $args{'rdir'}$args{'dir'}\n";

	if (-e $MD5_EXE) {
		my $md5;
		$md5 = `${TCTDIR}icat -H $img $inode | $MD5_EXE`;
		chop $md5;
		print "MD5 of file: $md5\n";
		if ($sort == $FIL_SORT_STR) {
			$md5 = `${TCTDIR}icat -H $img $inode | $STRINGS_EXE | $MD5_EXE`;
			chop $md5;
			print "MD5 of strings: $md5\n";
		}
	}

	print "Image: $img\n";

	my $date = localtime();
	print "Date Generated: $date\n".
		"Investigator: $INVESTIG\n".
		" ------------------------------------------------------\n";

	foreach $_ (`${TCTUTILDIR}istat -z $hr $img $inode`) {
        print "$_";
	}

	print "\n ------------------------------------------------------\n";

	if ($sort == $FIL_SORT_ASC) {
		print_output (`${TCTDIR}icat -H $img $inode`);
	} elsif ($sort == $FIL_SORT_STR) {
		print_output (`${TCTDIR}icat -H $img $inode | $STRINGS_EXE`);
	}

	return 0;
};


################# BLOCKS ##########################

# Generate the 2 frames for block browsing
$funcs[$BLK_MAIN] = sub {
	if ( check_img() || check_rdir()) {
		print_config();
		return 1;
	}

	print "<HEAD><TITLE>Block Browse on $args{'img'}</TITLE></HEAD>\n".
		"<FRAMESET COLS=\"20%,80%\">\n";

	# Block: Listings
	print "<FRAME SRC=\"$PROGNAME?func=$BLK_LIST&$baseargs\">\n";

	# Block Contents
	print "<FRAME SRC=\"$PROGNAME?func=$BLK_ENT&$baseargs\" ".
	  "NAME=\"content\">\n</FRAMESET>\n";

	return 0;
};

# Generate menu for blocks
$funcs[$BLK_LIST] = sub {
	if ( check_img() || check_rdir ()) {
		print_config();
		return 1;
	}

	print_index();

	print "<H3><A HREF=\"$PROGNAME?func=$BLK_ENT&$baseargs\"".
		" TARGET=\"content\">Enter Block Number</A></H3>\n".
		"<HR>\n";

	return 0;
};

# generate frame for block content
$funcs[$BLK_CIDXF] = sub {
	if (check_img() || check_block() || check_rdir()) {
		print_config();
		return 1;
	}

	print "<FRAMESET ROWS=\"20%,80%\">\n".
	  "<FRAME SRC=\"$PROGNAME?func=$BLK_CIDX&$baseargs".
	  "&block=$enc_args{'block'}&sort=$BLK_SORT_ASC\">\n".
	  "<FRAME SRC=\"$PROGNAME?func=$BLK_CONT&$baseargs".
	  "&block=$enc_args{'block'}&sort=$BLK_SORT_ASC\" NAME=\"cont2\">\n".
	  "</FRAMESET>";

	return 0;
};


# Generate index for block content
$funcs[$BLK_CIDX] = sub {
	if (check_img() || check_block() || check_bcat() || 
	  check_find_inode() || check_find_file() || check_rdir()) {
		print_config();
		return 1;
	}

	my $block = get_block();
	my $prev = $block - 1;
	my $next = $block + 1;

	print "<CENTER><FONT SIZE=3>";
	print "<A HREF=\"$PROGNAME?func=$BLK_CIDXF&$baseargs&block=$prev\"".
	  " TARGET=\"_parent\"><- previous</A>\n" unless ($prev < 0);

	print "<A HREF=\"$PROGNAME?func=$BLK_CIDXF&$baseargs&block=$next\"".
	  " TARGET=\"_parent\">next -></A>\n".
	  "<BR></FONT></CENTER>\n".
	  "<H3>Contents Of Block: $block<BR>\n";

	my $img = get_img();
	my $inode = `${TCTUTILDIR}find_inode $img $block`;

	if ($inode =~ /^(\d+)$/) {
		$inode = $1;
		my $tmpr = $args{'rdir'};
		if ($tmpr =~ /^(.*?)\/$/) {
			$tmpr = $1;
		}
		print "Belongs to inode: ".
		  "<A HREF=\"$PROGNAME?func=$INO_CONT&$baseargs&inode=$inode\"".
		  " TARGET=\"_parent\">$inode</A><BR>\n";

		print "<H4>";
		foreach $_ (`${TCTUTILDIR}find_file -a $img $inode`) {
			chop ;
			if (/(\*)\s+(.*)/) {
				print_output("$tmpr$2 (deleted)<BR>\n");
			} elsif (/(\/.*)/) {
				print_output("$tmpr$1<BR>\n");
			} else {
				print_output("$_<BR>\n");
			}
		}
		print "</H4>";
	}
	else {
		print "$inode\n";
	}

	print "<CENTER><BR><H4>".
	  "<A HREF=\"$PROGNAME?func=$BLK_CONT&$baseargs&sort=$BLK_SORT_ASC".
	  "&block=$block\" TARGET=\"cont2\"> Ascii </A>\n".
	  "<A HREF=\"$PROGNAME?func=$BLK_REP&$baseargs&sort=$BLK_SORT_ASC".
	  "&block=$block\" TARGET=\"_blank\">(report)</A> - \n".
	  "<A HREF=\"$PROGNAME?func=$BLK_CONT&$baseargs&sort=$BLK_SORT_HEX".
	  "&block=$block\" TARGET=\"cont2\"> Hex </A>\n".
	  "<A HREF=\"$PROGNAME?func=$BLK_REP&$baseargs&sort=$BLK_SORT_HEX".
	  "&block=$block\" TARGET=\"_blank\">(report)</A> - \n".
	  "<A HREF=\"$PROGNAME?func=$BLK_CONT&$baseargs&sort=$BLK_SORT_STR".
	  "&block=$block\" TARGET=\"cont2\"> Strings </A>\n".
	  "<A HREF=\"$PROGNAME?func=$BLK_REP&$baseargs&sort=$BLK_SORT_STR".
	  "&block=$block\" TARGET=\"_blank\">(report)</A>\n</H4></CENTER>\n";

	return 0;
};

#Display actual block content
$funcs[$BLK_CONT] = sub {
	if (check_img() || check_block() || check_bcat() || check_sort()) {
		return 1;
	}
	elsif (($args{'sort'} == $BLK_SORT_STR) && (check_strings())) {
		return 1;
	}


	my $block = get_block();
	my $img = get_img();
	my $str = "Contents of block: $block on image: $img\n\n\n";

	if ($args{'sort'} == $BLK_SORT_HEX) {
		print "Hex ".$str;
		print `${TCTUTILDIR}bcat -h $img $block 1024`;
	} 
	elsif ($args{'sort'} == $BLK_SORT_ASC) {
		print "ASCII ".$str;
		print `${TCTUTILDIR}bcat -a $img $block 1024`;
	}
	elsif ($args{'sort'} == $BLK_SORT_STR) {
		print "String ".$str;
		print `${TCTUTILDIR}bcat $img $block 1024 | $STRINGS_EXE`;
	}

	return 0;
};

$funcs[$BLK_REP] = sub {
	if (check_img() || check_block() || check_bcat() || check_sort()) {
		return 1;
	}
	elsif (($args{'sort'} == $BLK_SORT_STR) && (check_strings())) {
		return 1;
	}

    my $sort = $args{'sort'};
    my $img = get_img();
	my $block = get_block();
    my $type;

    if ($sort == $BLK_SORT_ASC) {
        $type = "ascii";
    } elsif ($sort == $BLK_SORT_STR) {
        $type = "string";
    } elsif ($sort == $BLK_SORT_HEX) {
        $type = "hex";
    } else {
        print "invalid sort value";
		print_config();
        return 1;
    }

    print "           $NAME $type Block Report (ver $VER)\n\n".
          " ------------------------------------------------------\n".
        "Block: $args{'block'}\n";

	my $inode = `${TCTUTILDIR}find_inode $img $block`;
	if ($inode =~ /^(\d+)$/) {
		my $tmpi = $1;
		print "Belongs to inode: $tmpi\n";

		my $tmpr = $args{'rdir'};
		if ($tmpr =~ /^(.*?)\/$/) {
			$tmpr = $1;
		}
		print "Belongs to files:\n";
		foreach $_ (`${TCTUTILDIR}find_file -a $img $tmpi`) {
			chop ;
			if (/(\*)\s+(.*)/) {
				print_output("  $tmpr$2 (deleted)\n");
			} elsif (/(\/.*)/) {
				print_output("  $tmpr$1\n");
			} else {
				print_output("  $_\n");
			}
		}
	} else {
		print "Not allocated to any inodes\n";
	}

    if (-e $MD5_EXE) {
        my $md5;
		$md5 = `${TCTUTILDIR}bcat $img $block 1024 | $MD5_EXE`;
        chop $md5;
        print "MD5 of raw block: $md5\n";

        if ($sort == $BLK_SORT_HEX) {
			$md5 = `${TCTUTILDIR}bcat -h $img $block 1024 | $MD5_EXE`;
		} elsif ($sort == $BLK_SORT_ASC) {
			$md5 = `${TCTUTILDIR}bcat -a $img $block 1024 | $MD5_EXE`;
		} elsif ($sort == $BLK_SORT_STR) {
			$md5 = `${TCTUTILDIR}bcat $img $block 1024 | $STRINGS_EXE | $MD5_EXE`;
		}
		chop $md5;
		print "MD5 of $type output: $md5\n";
    }

    print "Image: $img\n";

    my $date = localtime();
    print "Date Generated: $date\n".
        "Investigator: $INVESTIG\n".
        " ------------------------------------------------------\n";


	if ($sort == $BLK_SORT_HEX) {
		print_output (`${TCTUTILDIR}bcat -h $img $block 1024`);
	} 
	elsif ($sort == $BLK_SORT_ASC) {
		print_output(`${TCTUTILDIR}bcat -a $img $block 1024`);
	}
	elsif ($sort == $BLK_SORT_STR) {
		print_output(`${TCTUTILDIR}bcat $img $block 1024 | $STRINGS_EXE`);
	}

	return 0;
};

# Input form for block number
$funcs[$BLK_ENT] = sub {
	if ( check_img() || check_rdir()) {
		print_config();
		return 1;
	}

	print "<BODY><CENTER><H2>Block Display</H2>\n".
		"<FORM ACTION=\"$PROGNAME\" METHOD=\"GET\">\n".
		"Enter Block Number: <INPUT TYPE=\"text\" NAME=\"block\"><BR>\n".
		"<INPUT TYPE=\"hidden\" NAME=\"func\" VALUE=\"$BLK_CIDXF\">\n".
		"<INPUT TYPE=\"hidden\" NAME=\"img\" VALUE=\"$args{'img'}\">\n".
		"<INPUT TYPE=\"hidden\" NAME=\"rdir\" VALUE=\"$args{'rdir'}\">\n".
		"<INPUT TYPE=\"hidden\" NAME=\"hour\" VALUE=\"$args{'hour'}\">\n".
		"<INPUT TYPE=\"SUBMIT\" VALUE=\"Display\">\n".
		"</FORM></CENTER></BODY>\n";

	return 0;
};

########################### INODE ####################
$funcs[$INO_MAIN] = sub {
	if ( check_img() || check_rdir()) {
		print_config();
		return 1;
	}

	print "<HEAD><TITLE>Inode Browse on $args{'img'}</TITLE></HEAD>\n".
		"<FRAMESET COLS=\"20%,80%\">\n";

	# Inode Listings
	print "<FRAME SRC=\"$PROGNAME?func=$INO_LIST&$baseargs\">\n";

	# Inode Contents
	print "<FRAME SRC=\"$PROGNAME?func=$INO_ENT&$baseargs\" ".
	  "NAME=\"content\">\n</FRAMESET>\n";

	return 0;
};

# Generate lhs frame with menu and link to INO_ENT
$funcs[$INO_LIST] = sub {
	if ( check_img() || check_rdir() ) {
		print_config();
		return 1;
	}

	print_index();

	print "<H3><A HREF=\"$PROGNAME?func=$INO_ENT&$baseargs\"".
		" TARGET=\"content\">Enter Inode Number</A></H3>\n".
		"<HR>\n";

	return 0;
};

# Display the contents of inode
$funcs[$INO_CONT] = sub {
	if (check_img() || check_inode() ||check_istat() || check_find_file()
	  || check_rdir()) {
		return 1;
	}
	my $inode = get_inode();
	my $img = get_img();

	my $prev = $inode-1;
	my $next = $inode+1;

	print "<FONT SIZE=3><CENTER>\n";
	print "<A HREF=\"$PROGNAME?func=$INO_CONT&$baseargs&inode=$prev\">".
	  "<- previous</A>\n" unless ($prev < 2);
	print "<A HREF=\"$PROGNAME?func=$INO_CONT&$baseargs&inode=$next\">".
	  "next -></A>\n<BR>";

	print "<A HREF=\"$PROGNAME?func=$INO_REP&$baseargs&inode=$inode\"".
	  " TARGET=\"_blank\">inode report </A> - \n".
	  "<A HREF=\"$PROGNAME?func=$FIL_CIDXF&$baseargs&inode=$inode&dir=\">".
	  " view as file</A>\n".
	  "</FONT></CENTER>\n".
	  "<H3>Contents Of Inode: $inode<BR>\n".
	  "Belongs to files:<BR>\n"; 

	my $tmpr = $args{'rdir'};
	if ($tmpr =~ /^(.*?)\/$/) {
		$tmpr = $1;
	}

	foreach $_ (`${TCTUTILDIR}find_file -a $img $inode`) {
		chop ;
		if (/(\*)\s+(.*)/) {
			print_output("$tmpr$2 (deleted)<BR>\n");
		} elsif (/(\/.*)/) {
			print_output("$tmpr$1<BR>\n");
		} else {
			print_output("$_<BR>\n");
		}
	}
	print "</H3>\n";

	my $hr = get_hour();
	my $mode =0;

	foreach $_ (`${TCTUTILDIR}istat -z $hr $img $inode`) {
		if ($mode == 1) {
			my $blk;
			foreach $blk (split (/ /, $_)) {
				print "<A HREF=\"$PROGNAME?func=$BLK_CIDXF&$baseargs".
				  "&block=$blk\">$blk</A>  ";
			}
			print "<BR>\n";
		}
		else {
			print "$_<BR>\n";
			$mode = 1 if (/^Blocks/);
		}
	}

};

$funcs[$INO_REP] = sub {
	if (check_img() || check_inode() ||check_istat() || check_find_file()
	  || check_rdir()) {
		print_config();
		return 1;
	}

	my $inode = get_inode();
	my $img = get_img();
	my $hr = get_hour();

	print "           $NAME inode Report (ver $VER)\n\n".
		  " ------------------------------------------------------\n".
		"inode: $args{'inode'}\n";

	print "Belongs to files:\n"; 
	my $tmpr = $args{'rdir'};
	if ($tmpr =~ /^(.*?)\/$/) {
		$tmpr = $1;
	}
	foreach $_ (`${TCTUTILDIR}find_file -a $img $inode`) {
		chop ;
		if (/(\*)\s+(.*)/) {
			print_output("  $tmpr$2 (deleted)\n");
		} elsif (/(\/.*)/) {
			print_output("  $tmpr$1\n");
		} else {
			print_output("  $_\n");
		}
	}
	if (-e $MD5_EXE) {
		my $md5;
		$md5 = `${TCTUTILDIR}istat -z $hr $img $inode | $MD5_EXE`;
		chop $md5;
		print "MD5 of istat output: $md5\n";
	}

	print "Image: $img\n";

	my $date = localtime();
	print "Date Generated: $date\n".
		"Investigator: $INVESTIG\n".
		" ------------------------------------------------------\n";

	foreach $_ (`${TCTUTILDIR}istat -z $hr $img $inode`) {
        print "$_";
	}

	print "\n ------------------------------------------------------\n";

	return 0;
};

# Give a form for user to enter inode number to display
$funcs[$INO_ENT] = sub {
	if ( check_img() || check_rdir()) {
		print_config();
		return 1;
	}

	print "<BODY><CENTER><H2>Inode Display</H2>\n".
		"<FORM ACTION=\"$PROGNAME\" METHOD=\"GET\">\n".
		"Enter Inode Number: <INPUT TYPE=\"text\" NAME=\"inode\"><BR>\n".
		"<INPUT TYPE=\"hidden\" NAME=\"func\" VALUE=\"$INO_CONT\">\n".
		"<INPUT TYPE=\"hidden\" NAME=\"img\" VALUE=\"$args{'img'}\">\n".
		"<INPUT TYPE=\"hidden\" NAME=\"rdir\" VALUE=\"$args{'rdir'}\">\n".
		"<INPUT TYPE=\"hidden\" NAME=\"hour\" VALUE=\"$args{'hour'}\">\n".
		"<INPUT TYPE=\"SUBMIT\" VALUE=\"Display\">\n".
		"</FORM></CENTER></BODY>\n";

	return 0;
};


############ SEARCHING ##################

$funcs[$SRCH_MAIN] = sub {
	if ( check_img() || check_str() || check_rdir()) {
		print_config();
		return 1;
	}

	print "<HEAD><TITLE>Search on $args{'img'} for $args{'str'}".
	  "</TITLE></HEAD>\n".
	  "<FRAMESET COLS=\"20%,80%\">\n";

	# Block List
	print "<FRAME SRC=\"$PROGNAME?func=$SRCH_LIST&$baseargs".
	  "&str=$enc_args{'str'}\">\n";

	# Block Contents
	print "<FRAME SRC=\"$PROGNAME?func=$BLANK&$baseargs\" NAME=\"content\">\n".
		"</FRAMESET>\n";

	return 0;
};

# Form to enter search data
$funcs[$SRCH_ENT] = sub {
	if ( check_img() || check_rdir()) {
		print_config();
		return 1;
	}

	print "<HEAD><TITLE>Search on $args{'img'}</TITLE></HEAD>\n";

	print "<BODY><CENTER><H2>Image Search on $args{'img'}</H2>\n".
		"<FORM ACTION=\"$PROGNAME\" METHOD=\"GET\">\n".
		"Enter String: <INPUT TYPE=\"text\" NAME=\"str\"><BR>\n".
		"<INPUT TYPE=\"hidden\" NAME=\"func\" VALUE=\"$SRCH_MAIN\">\n".
		"<INPUT TYPE=\"hidden\" NAME=\"img\" VALUE=\"$enc_args{'img'}\">\n".
		"<INPUT TYPE=\"hidden\" NAME=\"rdir\" VALUE=\"$enc_args{'rdir'}\">\n".
		"<INPUT TYPE=\"hidden\" NAME=\"hour\" VALUE=\"$enc_args{'hour'}\">\n".
		"<INPUT TYPE=\"SUBMIT\" VALUE=\"Find\">\n".
		"</FORM></CENTER></BODY>\n";

	return 0;
};

# List of hits for search  (performs actual search)
$funcs[$SRCH_LIST] = sub {
	if ( check_img() || check_str() || check_rdir() || check_grep()) {
		print_config();
		return 1;
	}

	my $str = get_str();
	my $img = get_img();
	my $prev = -1;

	print "<HEAD><TITLE>Search for $str on $args{'img'}</TITLE></HEAD>\n";

	print_index();

	print "<H3><A HREF=\"$PROGNAME?func=$SRCH_ENT&$baseargs\"".
		"TARGET=\"_top\">New Search</A><BR>\n<HR>";

	print "$str found in blocks: </H3>\n";
	my @out;
	my $grep;
	if (-e $STRINGS_EXE) {
		@out = `$STRINGS_EXE -t d $img | $GREP_EXE \"$str\"`;
		$grep = 0;
	} else {
		@out = `$GREP_EXE -ab \"$str\" $img`;
		$grep = 1;
	}
	foreach $_ (@out) {
		if ((($grep == 1) && (/^(\d+):(.+)$/)) ||  
		  (($grep == 0) && (/^(\d+)\s+(.+)$/))) {
			my $idx = 0;
			while (($idx = index ($2, $str, $idx)) > -1) {
				my $b = ($1+$idx) / 1024;
				my $o = ($1+$idx) % 1024;

				if ($b != $prev) {
					my $url="$PROGNAME?func=$BLK_CIDXF&$baseargs&block=$b";

					print "$b (<A HREF=\"$url&sort=0\" TARGET=content>".
					  "Hex</A> - <A HREF=\"$url&sort=1\" TARGET=content>".
					  "Ascii</A>)<BR>";
					$prev = $b;
				}
				print "- offset $o bytes<BR>\n";
				$idx++;
			}
		}
		else {
			print "MISSED: $_<BR>\n";
		}
	}

	return 0;
};

##### Begin main ############

sub autopsy_main {
	# Parse arguments
	my $args = shift;

	foreach my $nam_val (split (/&/, $args)) {
		my ($name, $value) = split (/=/, $nam_val);
		if (defined $value) {
			$enc_args{$name} = $value;
			$args{$name} = url_decode($value);
		}
	}

	if ($TCTDIR !~ /.*?\/$/) {
		$TCTDIR .= '/';
	}

	if ($TCTUTILDIR !~ /.*?\/$/) {
		$TCTUTILDIR .= '/';
	}

	if ($MORGDIR !~ /.*?\/$/) {
		$MORGDIR .= '/';
	}

	if (check_func()) {
		# If no argument was given then show the menu
		if (exists $args{'func'}) {
			print_html_header();
			print "Invalid func argument (positive numbers only)<BR>\n";
			print_config();
		}
		else {
			print_html_header();

			print "<META HTTP-EQUIV=\"Refresh\" CONTENT=\"0; ".
			  "URL=$PROGNAME?func=$MENU\">";
			print "<H3><A HREF=\"$PROGNAME?func=$MENU\">Menu</A>"; 
		}
	}

	elsif (($args{'func'} == $FIL_CONT) || 
	  ($args{'func'} == $FIL_REP) ||
	  ($args{'func'} == $BLK_CONT) ||
	  ($args{'func'} == $BLK_REP) ||
	  ($args{'func'} == $INO_REP)) {
		print_text_header();
		& {$funcs[$args{'func'}]};
	}
	else {
		unless (($args{'func'} == $MENU) || ($args{'func'} == $HELP)) {
			$baseargs = "rdir=$enc_args{'rdir'}&img=$enc_args{'img'}".
			  "&hour=$enc_args{'hour'}";
		}

		print_html_header();
		& {$funcs[$args{'func'}]};
		print "</HTML>\n";
	}

	return 0;
}

#### END OF MAIN ####


###################### INPUT CHECKS ######################3
sub check_func {
	if ((!exists $args{'func'}) || ($args{'func'} !~ /^\d+$/) ||
 	  (!(defined $funcs[$args{'func'}])) )  {

		return 1;
	}
	return 0;
}

# We don't allow much for the image because this is an argument to
# the TCT and TCTUTILs programs.  We keep these files only in one
# directory and for easy/simple security only allow basic names
# Symbolic links are allowed if these simple names are not desired
#
# Allowed values are A-Za-z0-9_-.
sub check_img {
	if (!exists $args{'img'}) {
		print "Missing image argument (did you check both boxes on the ".
		  "main menu?)<BR>\n";
		return 1;

	} elsif (($args{'img'} !~ /^[\w\-\_\.]+$/) ||
	  ($args{'img'} eq "")) {
		print "Invalid image argument (only letters, numbers,-,., and _)<BR>\n";
		return 1;
	}
	# Check for its actual existence
	if ($args{'img'} =~ /^([\w\-\_]+)$/) {
		unless (-e $MORGDIR.$1) {
			print "Missing image: $MORGDIR$1\n";
			return 1;
		}
	}
	return 0;
}

sub get_img {
	if ($args{'img'} =~ /^([\w\-\_\.]+)$/) {
		return $MORGDIR.$1;
	}
	return 0;
}


# dir is only used for printing purposes so we don't have to be concerned
# about "bad" characters.  This also allows us to show directories that have
# names with spaces and back spaces in them
sub check_dir {
	if ((!exists $args{'dir'}) ||
	  ($args{'dir'} =~ /\/\.\.\//) ) {
		print "Invalid dir argument (valid file path only)<BR>\n";
		return 1;
	}
	return 0;
}

sub check_rdir {
	if (!exists $args{'rdir'}) {
		print "Missing rdir argument ".
		  "(did you check both boxes on the main menu?)<BR>\n";
		return 1;
	}
	elsif ($args{'rdir'} =~ /\/\.\.\//) {
		print "Invalid rdir argument (valid file path only)<BR>\n";
		return 1;
	}
	return 0;
}

sub check_inode {
	if ((!exists $args{'inode'}) || ($args{'inode'} !~ /^\d+$/) ||
		($args{'inode'} <= 0)) {
		print "Invalid inode argument (numbers > 0 only)<BR>\n";
		return 1;
	}
	return 0;
}

# We can only have positive numbers
sub get_inode {
	if ($args{'inode'} =~ /^(\d+)$/) {
		return $1;
	}
	return 0;
}

sub check_block {
	if ((!exists $args{'block'}) || ($args{'block'} !~ /^\d+$/)) {
		print "Invalid block argument (positive numbers only)<BR>\n";
		return 1;
	}
	return 0;
}

sub check_sort {
	if ((!exists $args{'sort'}) || ($args{'sort'} !~ /^\d+$/)) {
		print "Invalid sort argument (positive numbers only)<BR>\n";
		return 1;
	}
	return 0;
}

sub get_block {
	if ($args{'block'} =~ /^(\d+)$/) {
		return $1;
	}
	return 0;
}

# This should be made more flexible
sub check_str {
	if ((!exists $args{'str'}) || 
	  ($args{'str'} !~ /^\s*([\d\w\.\-\_\"\'\s]+)$/)) {
		print "Invalid string argument (letters, numbers, white space, ., -, _, \", \' only)<BR>\n";
		return 1;
	}
	return 0;
}

# Get the string to search for with grep
# This should be made more flexible
sub get_str {
	if ($args{'str'} =~ /^\s*([\d\w\.\-\_\"\'\s]+)$/) {
		return $1;
	}
	return 0;
}

sub get_hour {
	if ($args{'hour'} =~ /^([-+]?\d+)$/) {
		return $1;
	}
	return 0;
}


# Check for executables
sub check_icat {
	unless (-x ${TCTDIR}.'icat') {
		print "TCT icat executable missing<br>\n";
		return 1;
	}
	return 0;
}

sub check_istat {
	unless (-x ${TCTUTILDIR}.'istat') {
		print "TCTUTILs istat executable missing<br>\n";
		return 1;
	}
	return 0;
}

sub check_fls {
	unless (-x ${TCTUTILDIR}.'fls') {
		print "TCTUTILs fls executable missing<br>\n";
		return 1;
	}
	return 0;
}

sub check_bcat {
	unless (-x ${TCTUTILDIR}.'bcat') {
		print "TCTUTILs bcat executable missing<br>\n";
		return 1;
	}
	return 0;
}

sub check_find_file {
	unless (-x ${TCTUTILDIR}.'find_file') {
		print "TCTUTILs find_file executable missing<br>\n";
		return 1;
	}
	return 0;
}

sub check_find_inode {
	unless (-x ${TCTUTILDIR}.'find_inode') {
		print "TCTUTILs find_inode executable missing<br>\n";
		return 1;
	}
	return 0;
}

sub check_fsmorgue {
	unless (-d $MORGDIR) {
		print "Error Finding $MORGDIR<BR>\n";
		return 1;
	}
	unless (-f $MORGDIR.'fsmorgue') {
		print "${MORGDIR}fsmorgue file missing<br>\n";
		return 1;
	}
	return 0;
}

sub check_strings {
	unless (-x $STRINGS_EXE) {
		print "$STRINGS_EXE executable missing<br>\n";
		return 1;
	}
	return 0;
}

sub check_grep {
	unless (-x $GREP_EXE) {
		print "$GREP_EXE executable missing<br>\n";
		return 1;
	}
	return 0;
}



############## URL UTILITIES ##################

sub url_encode {
	my $text = shift;
	$text =~ s/([^a-z0-9_.!~*'() -])/sprintf "%%%02X", ord($1)/eig;
	$text =~ tr/ /+/;
	return $text;
}

sub url_decode {
	my $text = shift;
	$text =~ tr/\+/ /;
	$text =~ s/%([a-f0-9][a-f0-9])/chr( hex( $1 ) )/eig;
	return $text;
}

############ DISPLAY ########################
sub print_html_header {
	print "Content-type: text/html\n\n".
		"<HTML>\n".
		"<!-- $NAME ver. $VER Forensic Broswer -->\n".
		"<!-- ".localtime()." -->\n";
}

sub print_text_header {
	print "Content-type: text/plain\n\n";
}

# remove control chars from printout
sub print_output {
	while (my $out = shift) {
		foreach $_ (split (//, $out)) {
			if (($_ eq "\n") || ($_ eq "\r") || ($_ eq "\f") ||
			  ($_ eq "\t") ) {
				print "$_";
			}
			elsif ((ord($_) < 0x20) || (ord($_) > 0x7e)) {
				print "^".ord($_);
			}
			else {
				print "$_";
			}
		}
	}
}

# Print links to main menu, blocks, files etc for use in left frame
sub print_index {
	print "<H3><A HREF=\"$PROGNAME?func=$MENU&hour=$enc_args{'hour'}\"".
		"TARGET=\"_top\">Main Menu</A><BR>".
		"<A HREF=\"$PROGNAME?func=$FIL_MAIN&$baseargs&inode=2&dir=\"".
		"TARGET=\"_top\">Files</A><BR>".
		"<A HREF=\"$PROGNAME?func=$INO_MAIN&$baseargs\"".
		" TARGET=\"_top\">Inode</A><BR>".
		"<A HREF=\"$PROGNAME?func=$BLK_MAIN&$baseargs\"".
		" TARGET=\"_top\">Blocks</A><BR>".
		"<A HREF=\"$PROGNAME?func=$SRCH_ENT&$baseargs\"".
		" TARGET=\"_top\">Search</A><BR>".
		"</H3>\n<HR>\n";
}

